/*###############################################################################
# Linux Management Providers (LMP), SSH provider package
# Copyright (C) 2010 Feng Zhanlei, REDFLAG <fengzhanlei@redflag-linux.com>
# 
# This program is being developed under the "OpenDRIM" project.
# The "OpenDRIM" project web page: http://opendrim.sourceforge.net
# The "OpenDRIM" project mailing list: opendrim@googlegroups.com
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2
# of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#################################################################################

#################################################################################
# To contributors, please leave your contact information in this section
# AND comment your changes in the source code.
# 
# Modified by Guillaume BOTTEX <guillaumebottex@uxsystem.net>, UXSystem, 2010
###############################################################################*/

#include "cmpiOpenDRIM_SSHSettingData.h"
#include "OpenDRIM_SSHSettingDataAccess.h"

#include <iostream>

#define ToString(name, value, condition) os << (string) "      " + name + (string) "="; instance.condition ? os << "NULL" : os << instance.value; os << endl
#define ToStringInt(name, value, condition) os << (string) "      " + name + (string) "="; instance.condition ? os << "NULL" : os << (long) instance.value; os << endl
#define ToStringRef(name, value, condition) os << (string) "      " + name + (string) "="; instance.condition ? os << "NULL" : os << instance.value.toString(); os << endl
#define ToStringArray(name, value, condition) os << (string) "      " + name + (string) "=["; for (size_t i = 0; i < instance.value.size(); i++) { os << instance.value[i]; if (i < instance.value.size()-1) os << ","; } os << "]" << endl
#define ToStringArrayInt(name, value, condition) os << (string) "      " + name + (string) "=["; for (size_t i = 0; i < instance.value.size(); i++) { os << (long) instance.value[i]; if (i < instance.value.size()-1) os << ","; } os << "]" << endl
#define ToStringRefArray(name, value, condition) os << (string) "      " + name + (string) "=["; for (size_t i = 0; i < instance.value.size(); i++) { os << instance.value[i].toString(); if (i < instance.value.size()-1) os << ","; } os << "]" << endl

ostream& operator<<(ostream& os, OpenDRIM_SSHSettingData& instance) {
	os << "   [OpenDRIM_SSHSettingData]" << endl;
	ToString("Caption", Caption, Caption_isNULL);
	ToString("Description", Description, Description_isNULL);
	ToString("Generation", Generation, Generation_isNULL);
	ToString("InstanceID", InstanceID, InstanceID_isNULL);
	ToString("ElementName", ElementName, ElementName_isNULL);
	ToString("ConfigurationName", ConfigurationName, ConfigurationName_isNULL);
	ToStringInt("ChangeableType", ChangeableType, ChangeableType_isNULL);
	ToStringArrayInt("EnabledSSHVersions", EnabledSSHVersions, EnabledSSHVersions_isNULL);
	ToString("OtherEnabledSSHVersion", OtherEnabledSSHVersion, OtherEnabledSSHVersion_isNULL);
	ToStringInt("SSHVersion", SSHVersion, SSHVersion_isNULL);
	ToStringArray("OtherSSHVersion", OtherSSHVersion, OtherSSHVersion_isNULL);
	ToStringArrayInt("EnabledEncryptionAlgorithms", EnabledEncryptionAlgorithms, EnabledEncryptionAlgorithms_isNULL);
	ToStringArray("OtherEnabledEncryptionAlgorithm", OtherEnabledEncryptionAlgorithm, OtherEnabledEncryptionAlgorithm_isNULL);
	ToStringInt("EncryptionAlgorithm", EncryptionAlgorithm, EncryptionAlgorithm_isNULL);
	ToString("OtherEncryptionAlgorithm", OtherEncryptionAlgorithm, OtherEncryptionAlgorithm_isNULL);
	ToString("IdleTimeout", IdleTimeout, IdleTimeout_isNULL);
	ToString("KeepAlive", KeepAlive, KeepAlive_isNULL);
	ToString("ForwardX11", ForwardX11, ForwardX11_isNULL);
	ToString("Compression", Compression, Compression_isNULL);
	os << "   [\\OpenDRIM_SSHSettingData]" << endl;
	return os;
}

int main() {
	
	string errorMessage;

	int errorCode = SSH_OpenDRIM_SSHSettingData_load(NULL, errorMessage);
	if (errorCode != OK) {
		cout << "ERROR " << errorCode << ": " << errorMessage << endl;
		return -1;
	}

	cout << endl << "[enumInstances]" << endl;
	vector<OpenDRIM_SSHSettingData> instances;
	
	errorCode = SSH_OpenDRIM_SSHSettingData_retrieve(NULL, NULL, instances, NULL, errorMessage, "ei");
	if (errorCode != OK) {
		cout << "ERROR " << errorCode << ": " << errorMessage << endl;
		return -1;
	}
	
	vector<OpenDRIM_SSHSettingData>::iterator it = instances.begin();
	for (; it != instances.end(); ++it) {
		cout << *it;
	}
	
	cout << "[\\enumInstances]" << endl;
	cout << endl << "[enumInstanceNames & getInstance]" << endl;
	
	instances.clear();
	errorCode = SSH_OpenDRIM_SSHSettingData_retrieve(NULL, NULL, instances, NULL, errorMessage, "ein");
	if (errorCode != OK) {
		cout << "ERROR " << errorCode << ": " << errorMessage << endl;
		return -1;
	}
	
	it = instances.begin();
	for (; it != instances.end(); ++it) {
		errorCode = SSH_OpenDRIM_SSHSettingData_getInstance(NULL, NULL, *it, NULL, errorMessage);
		if (errorCode != OK) {
			cout << "ERROR " << errorCode << ": " << errorMessage << endl;
			return -1;
		}
		cout << *it;
	}
	
	cout << "[\\enumInstanceNames & getInstance]" << endl;
	cout << endl;

	errorCode = SSH_OpenDRIM_SSHSettingData_unload(errorMessage);
	if (errorCode != OK) {
		cout << "ERROR " << errorCode << ": " << errorMessage << endl;
		return -1;
	}
	return 0;
}
